// -*-c++-*-
/*
The MIT License (MIT)

Copyright (c) 2014 J. Eric Townsend

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/* basic stats with analogRead()
 * 
 * intputs
 * A0 photoresitro
 *
 * output
 * D13 LED
 * 40 NEOPIXEL
 *
 */

#include <Adafruit_NeoPixel.h>

#ifdef __AVR__
  #include <avr/power.h>
#endif
#define PIN            40
#define NUMPIXELS      1
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRBW + NEO_KHZ800);

static const int photoPin = 0;
static const int sampleSize = 16;
int readings[sampleSize];
int sampleIndex = 0;

static const int statusLed = 13;
int lastRedVal = 0;

void setup() {
    Serial.begin(115200);

    pinMode(photoPin, INPUT);
    
    pinMode(statusLed, OUTPUT);

    pixels.begin(); // This initializes the NeoPixel library.
}

void loop() {


    int photo = analogRead(photoPin);
    readings[sampleIndex] = photo;
    sampleIndex < (sampleSize - 1) ? sampleIndex++ : sampleIndex = 0;

    digitalWrite(photoPin, HIGH);
//    Serial.println(photo);
    int median = 0;
    int low = 1024;
    int high = 0;
    int mean = 0;
    for (int i = 0; i < sampleSize; i++) {
        mean += readings[i];
        if (readings[i] < low) {
            low = readings[i];
        }
        if (readings[i] > high) {
            high = readings[i];
        }
    }
    mean = mean / sampleSize;
    bubbleSort();
    median = readings[sampleSize / 2];
  
// console
#if 0
    Serial.print("low ");
    Serial.print(low);
    Serial.print(" high ");
    Serial.print(high);
    Serial.print(" mean ");
    Serial.println(mean);
    Serial.print(" median ");
    Serial.println(median);
    delay(500);
    digitalWrite(photoPin, LOW);
    delay(500);
#endif
#if 1
// plotter
    Serial.print(low);
    Serial.print(",");
    Serial.print(high);
    Serial.print(",");
    Serial.print(mean);
    Serial.print(",");
    Serial.println(median);
#endif
    
    // TODO
    // set the pixels with different values, mean, median, current read

// long map(long x, long in_min, long in_max, long out_min, long out_max)
// in A10, the value of 600 was the bottom end
    int redVal = map(median, 600, 1023, 0, 255);

// at home, 950 was the bottom end. Remember that higher value == less light
//    int redVal = map(median, 950, 1023, 0, 255);

// keep the pixel from blinking

/*
// simple, but allows for some blinking    
    if (redVal != lastRedVal) {
        pixels.setPixelColor(0, pixels.Color(redVal,0,0));
        pixels.show(); // This sends the updated pixel color to the hardware.
        lastRedVal = redVal;
    }
 */

    
 // use a tolerance to limit to a range
  // by making this not a const we can change it in the code
  // as needed 
    
    int tolerance = 5;

    // This looks like it should work, right?
    // what happens if lastRedVal is 0?
    if (
        (redVal >  (lastRedVal + tolerance))
        || (redVal <  (lastRedVal - tolerance))
        ) {
    
        pixels.setPixelColor(0, pixels.Color(redVal,0,0));
        pixels.show(); // This sends the updated pixel color to the hardware.
        lastRedVal = redVal;
    }
}


// tigoe's sort for an array
void bubbleSort() {
  int out, in, swapper;
  for(out=0 ; out < sampleSize; out++) {  // outer loop
    for(in=out; in<(sampleSize-1); in++)  {  // inner loop
      if( readings[in] > readings[in+1] ) {   // out of order?
        // swap them:
        swapper = readings[in];
        readings [in] = readings[in+1];
        readings[in+1] = swapper;
      }
    }
  }
}
