// -*-c++-*-
/*
The MIT License (MIT)

Copyright (c) 2014 J. Eric Townsend

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/* basic stats with analogRead()
 * 
 * intputs
 * A0 photoresitro
 *
 * output
 * D13 LED
 * 40 NEOPIXEL
 *
 */

#include <Adafruit_NeoPixel.h>

#ifdef __AVR__
  #include <avr/power.h>
#endif
#define PIN            40
#define NUMPIXELS      1
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRBW + NEO_KHZ800);

static const int photoPin = 0;
static const int sampleSize = 16;
int readings[sampleSize];
unsigned int sampleIndex = 0;
unsigned long lastSampleTime = 0;
unsigned long sampleInterval = 100; // in ms
// we need to move these out of the loop() as well
unsigned int median = 0;
unsigned int low = 1024;
unsigned int high = 0;
unsigned int mean = 0;

static const int statusLed = 13;
unsigned int lastRedVal = 0;

void setup() {
    Serial.begin(115200);

    pinMode(photoPin, INPUT);
    
    pinMode(statusLed, OUTPUT);

    pixels.begin(); // This initializes the NeoPixel library.
}

void loop() {
    // TODO what happens if you try to set a negative color?
    unsigned int redVal = 0;

    // TODO 
    // don't read the input on every loop to help stabilize the data
    // sampling over time is also a UI change, you have to actively
    // block light for
    // sampleSize * sampleInterval milliseconds to create a change

    unsigned long now = millis();
    if (lastSampleTime + sampleInterval < now) {
        lastSampleTime = now;
        unsigned int photo = analogRead(photoPin);

        // good for debugging:
        Serial.print(now);
        Serial.print(" ");
        Serial.println(photo);

        readings[sampleIndex] = photo;
        sampleIndex < (sampleSize - 1) ? sampleIndex++ : sampleIndex = 0;
        
        digitalWrite(photoPin, HIGH);
        for (int i = 0; i < sampleSize; i++) {
            mean += readings[i];
            if (readings[i] < low) {
                low = readings[i];
            }
            if (readings[i] > high) {
                high = readings[i];
            }
        }
        mean = mean / sampleSize;
        bubbleSort();
        median = readings[sampleSize / 2];
    }
  
    
    // TODO
    // set the pixels with different values, mean, median, current read
    
// long map(long x, long in_min, long in_max, long out_min, long out_max)
// in A10, the value of 600 was the bottom end
// TODO use a flashlight to break this, more light == lower number
// than 600 
    redVal = map(median, 600, 1023, 0, 255);

// TODO
//  at home, 950 was the bottom end. Remember that higher value == less light
// how do we dynamically set the bottom end of the map?
//    redVal = map(median, 950, 1023, 0, 255);

// TODO
// what about changing the max value of redVal so we have a dimmer range?
//    redVal = map(median, 600, 1023, 0, 128);
    




// TODO keep the pixel from blinking
/*
// simple, but allows for some blinking    
    if (redVal != lastRedVal) {
        pixels.setPixelColor(0, pixels.Color(redVal,0,0));
        pixels.show(); // This sends the updated pixel color to the hardware.
        lastRedVal = redVal;
    }
 */

    
 // use a tolerance to limit to a range
  // by making this not a const we can change it in the code
  // as needed 
    
    int tolerance = 5;

    // This looks like it should work, right?
    // what happens if lastRedVal is 0?
    if (
        (redVal >  (lastRedVal + tolerance))
        || (redVal <  (lastRedVal - tolerance))
        ) {
    
        pixels.setPixelColor(0, pixels.Color(redVal,0,0));
        pixels.show(); // This sends the updated pixel color to the hardware.
        lastRedVal = redVal;
    }

// console
#if 0
    Serial.print("low ");
    Serial.print(low);
    Serial.print(" high ");
    Serial.print(high);
    Serial.print(" mean ");
    Serial.println(mean);
    Serial.print(" median ");
    Serial.print(median);
    Serial.print(" red ");
    Serial.println(redVal);
    delay(500);
    digitalWrite(photoPin, LOW);
    delay(500);
#endif
#if 0
// plotter
    Serial.print(low);
    Serial.print(",");
    Serial.print(high);
    Serial.print(",");
    Serial.print(mean);
    Serial.print(",");
//    Serial.println(median);
    Serial.print(median);
    Serial.print(",");
    Serial.println(redVal);
#endif

}




// tigoe's sort for an array
void bubbleSort() {
  int out, in, swapper;
  for(out=0 ; out < sampleSize; out++) {  // outer loop
    for(in=out; in<(sampleSize-1); in++)  {  // inner loop
      if( readings[in] > readings[in+1] ) {   // out of order?
        // swap them:
        swapper = readings[in];
        readings [in] = readings[in+1];
        readings[in+1] = swapper;
      }
    }
  }
}
